#! /usr/bin/env python

"""
HTML preprocesser - written by Frederic Peters <fpeters@swing.be> 

Copyright (c) 1999-2000 Frederic Peters;

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 You will find a copy of the license on http://www.fsf.org

-- 

this is a simple (?) script I use to generate pages of Gaby website
(http://gaby.netpedia.net); it has several nice features but relies
heavily on my current directory layout:
 (dirs are relative to /home/fred/src/gaby (root of Gaby source tree))
 ./website/templates/ChangeLog -> ../../ChangeLog
 ./website/templates/NEWS -> ../../NEWS
 ./website/archives (filled with gaby-*.tar.gz and gaby-patch-*.gz)
 ./website/templates/*.html (body of real pages)
 ./website/templates/header.htm (top of real pages)
 ./website/templates/bottom.htm (bottom of real pages)
 ./website/templates/create.py (this script)
 ./website/index.html -> What.html

it shouldn't be difficult to adapt this for other programs;

body (*.html), header.htm and bottom.htm are distributed separately
"""

import os
import re
import time
import sys
import string
import find
from stat import *

def CreateChangeLog():
	if os.stat('ChangeLog.html')[8] >= os.stat('ChangeLog')[8]:
		return 1
	changelog = open('ChangeLog')
	inhtml = open('ChangeLog.html', 'w')
	line = changelog.readline()
	inhtml.write('<h1>ChangeLog</h1>\n\n')
#	inhtml.write('<p>\nThis file is automagically generated from the '\
#			'ChangeLog file distributed in the tarball.')
	while ( len(line) > 0 ):
		line = re.sub('<', '&lt;', line)
		line = re.sub('>', '&gt;', line)
		if ( line[0] == '\t' or ( len(line) > 1 and line[1] == '\t')  ):
			if ( len(line[3:-1]) == 0 ):
				inhtml.write('<br>\n')
			else:
				if ( line[1] == '*' ):
					if ( firsttime != 1 ):
						inhtml.write('<br>\n')
					firsttime = 0
					inhtml.write(line[3:-1])
				elif ( line[1] == ' '):
					inhtml.write(line[3:-1])
				else:
					inhtml.write(line[1:-1])
				inhtml.write(' ')
			line = changelog.readline()
			continue
		if ( len(line) == 1 ):
			line = changelog.readline()
			if ( line[3:11] == 'Released' ):
				inhtml.write( \
					'<a name="' + line[12:-1] + '"></a>')
			continue
		inhtml.write('\n</p>\n')
		inhtml.write('\t<h4>')
		inhtml.write(line[:-1])
		inhtml.write('</h4>\n')
		line = changelog.readline()
		inhtml.write('<p>\n')
		re.sub('<', '&lesser', line)
		re.sub('>', '&greater', line)
		firsttime = 1
	inhtml.write('\n</p>\n')
	inhtml.close()
	changelog.close()
	return 0

def CreateNews():
	if os.stat('Releases.html')[8] >= os.stat('NEWS')[8]:
		return 1
	news = open('NEWS')
	inhtml = open('Releases.html', 'w')

	inhtml.write('<h1>Releases</h1>\n\n')
	inhtml.write('<p>Tarballs and incremental patches are available on '\
		     'the <a href="Download.html">downloads</a> page.</p>\n\n')
	line = news.readline()
	text = 0
	while ( len(line) != 0 ):
		if ( len(line) == 1 ):
			if ( text == 2 ):
				inhtml.write('</p>\n')
				if ( version != '1.9.0' ):
					inhtml.write('<p>Read the <a href="ChangeLog.html#')
					inhtml.write(version)
					inhtml.write('">ChangeLog</a></p>\n\n')
				text = 0
			if ( text == 1 ):
				inhtml.write('<p>')
				text = 2
		if ( line[:4] == '--- ' and '?' not in line ):
			text = 1
			p = string.split(line, ' ')
			version = p[1]
			inhtml.write('<h4><a name="' + p[1] + '">' + p[1] + \
				' - ' + \
				string.join((p[3],p[4],p[5][:-1]), ' ') + \
				'</a></h4>\n\n' )
		if ( text == 2 ):
			inhtml.write(line)
		
		line = news.readline()
		
	inhtml.close()
	news.close()
	return 0

def get_gaby_version():
	p = os.popen('cat config.h | grep VERSION | cut -d \' \' -f 3')
		# the shell might still be useful :)
	version = p.read()
	p.close()
	return version[1:-2]

def SizeOf(s):
	try:
		size = os.stat(s)[ST_SIZE]
	except OSError:
		size = 0
	unit = 'b '
	if ( size > 1024 ):
		size = size / 1024
		unit = 'k '
	size = repr(size) + unit
	return size

def vcmp(s1, s2):
	def conv(s):
		v1 = string.split(s, '-')[2]
		vs1 = string.split(v1, '.')
		v1 = string.atoi(vs1[0])*10000 + \
			string.atoi(vs1[1])*100 + string.atoi(vs1[2])
		return v1
	return cmp( conv(s1), conv(s2))

def PatchSection():
	lines = '<!-- auto-generated -->\n<ul>\n'
	patches = find.find('gaby-patch*', '../archives')
	def cut(s): return s[12:]
	patches = map(cut, patches)
	patches.sort( lambda x,y: - vcmp(x, y))
	i = 0
	while ( i < len(patches) ):
		newline = patches[i][11:-3]
		newline = re.sub('-', ' to ', newline)
		size = SizeOf('../archives/' + patches[i])
		newline = '<li><a href="archives/' + patches[i] + \
			'">' + newline + '</a> (' + size[:-1] + ')</li>\n'
		lines = lines + newline
		i = i+1
	lines = lines + '</ul>\n'
	return lines

def ImportFile(s):
	f = open(s)
	lines = f.readlines()
	f.close()
	longline = '<!-- section imported from ' + s + '-->\n'
	for line in lines:
		longline = longline + line
	longline = longline + '\n<!-- end of section -->\n'
	return longline

def GetLastNewsItem():
	file = open('News.html')
	s = file.readline()
	while ( s[:4] != '<h4>'):
		s = file.readline()
	file.close()
	if ( '(' in s ):
		p2 = string.index(s, '(')-1
	else:
		p2 = -6
#	return s[string.index(s, '-')+2:p2]
	return s[4:p2]

def WriteFull(s):
	if ( len(sys.argv) >= 2 ):
		headerfile = 'header.' + sys.argv[1] + '.htm'
		bottomfile = 'bottom.' + sys.argv[1] + '.htm'
	else:
		headerfile = 'header.htm'
		bottomfile = 'bottom.htm'

	titles = { 'What':'What is Gaby ?', 'index':'Welcome !' }
	if ( titles.has_key(s[:-5]) ):
		title = titles[s[:-5]]
	else:
		title = s[:-5]
	
	date = time.strftime('%a, %-d %b %Y %H:%M:%S', 
					time.gmtime(os.stat(s)[ST_MTIME]) )
	
	version = get_gaby_version()
	newsitem = GetLastNewsItem()

	newfilename = '../' + s
	file = open(newfilename, 'w')
	for name in [ headerfile, s, bottomfile]:
		f = open(name)
		fc = f.readlines()
		for i in range(0, len(fc)):
			fc[i] = re.sub('TITLE', title, fc[i])
			fc[i] = re.sub('DATE', date, fc[i])
			fc[i] = re.sub('VERSION', version, fc[i])
			fc[i] = re.sub('LATEST NEWS ITEM', newsitem, fc[i])
			if ( fc[i][:7] == 'SIZE OF' ):
				fc[i] = SizeOf('../' + fc[i][8:-1])
			if ( fc[i][:-1] == 'PATCHES' ):
				fc[i] = PatchSection()
			if ( fc[i][:6] == 'IMPORT' ):
				fc[i] = ImportFile(fc[i][7:-1])
		f.close()
		file.writelines(fc)
	file.close()

def main():
	print 'Creating ChangeLog.html (from ChangeLog)',
	if CreateChangeLog():	print '(skipped since it was up-to-date)'
	else:			print ''
	
	print 'Creating Releases.html (from NEWS)',
	if CreateNews():	print '(skipped since it was up-to-date)'
	else:			print ''
	
	def f(s): return s[-5:] == '.html'
	files = filter(f, os.listdir('.'))
	for s in files:
		if os.stat('../'+s)[8] > os.stat(s)[8]:
			print 'Skipping', s
			continue
		print 'Working on', s
		WriteFull(s)

main()

